#############################################
# update_site.py - wxWidgets web site updater
# Author: Kevin Ollivier
# License: wxWidgets License
#############################################

""" This module allows us to use non-destructive templates. That means that you do not need to re-generate pages anytime a page changes - only when you change a template. Whenever you alter a template, run this script and it will apply the updates to the whole site.
"""

import sys, os, string, re
import PyMeld
import copy
from section_links import *

root_dir = os.path.abspath(os.path.join("..", "site-v2"))
    
print "Loading templates..."
# First, read in the site template(s)
main_template = open(os.path.join(root_dir, "Templates", "template.dwt")).read()
template_meld = PyMeld.Meld(main_template)

section_file = os.path.join(root_dir, "Templates", "section_template.dwt")
section_template = open(section_file).read()
section_meld = PyMeld.Meld(section_template)

# Update the section template with any changes to the main template
sec_template = copy.deepcopy(template_meld)
sec_template.contents._content = section_meld.contents._content
section_meld = sec_template        

def updateDir(dirname):
    """
    This function recursively walks through the subdirectories and
    updates any HTML files it finds.
    """
    for item in os.listdir(dirname):
        fullpath = os.path.join(dirname, item)
        
        if os.path.isfile(fullpath):
            extension = os.path.splitext(fullpath)[1]
            if string.find(extension, "htm") != -1 or string.find(extension, "php") != -1:
                updateFile(fullpath)
        
        elif os.path.isdir(fullpath):
            updateDir(fullpath)
            
def adjustDWTemplate(page_template):
    """
    DreamWeaver templates and instances of templates use slightly different syntax. Adjust
    the template macros to reflect the names of instrance macros.
    """
    page_template = page_template.replace("TemplateBeginEditable", "InstanceBeginEditable")
    page_template = page_template.replace("TemplateEndEditable", "InstanceEndEditable")
    
    return page_template
    
                        
def updateFile(filename):
    """
    This imports the existing page contents (minus the template areas) and
    then applies the latest template to those contents.
    """
    try:
        # Read in the page data we want to keep
        html = open(filename).read()
        title_regex = re.compile("<title>([^<]*)</title>", re.IGNORECASE)
        html_regex = re.compile("<html>(.*)</html>", re.IGNORECASE | re.DOTALL)
        title = title_regex.search(html).group(0)
        html_meld = PyMeld.Meld(html)
        
        try:
            contents = html_meld.contents._content
        except:
            print "File does not appear to use template: " + filename
            return
        
        print "Updating %s" % (filename)
        
        # Check if it's a sub-section of the site, and if so, add the links
        relative_path = string.replace(os.path.dirname(filename), root_dir, "")[1:]
        
        # Make a copy of the main template, then insert the new values
        global template_meld
        page_template = copy.deepcopy(template_meld)
        
        if relative_path != "":
            section = relative_path.split(os.sep)[0]
            mylinks = links[section]
            global section_meld
            page_template = copy.deepcopy(section_meld)
            
            section_contents = html_meld.section_contents._content
            sec_regex = re.compile('<!-- InstanceBeginEditable name="section_contents" -->(.*)<!-- InstanceEndEditable -->', re.IGNORECASE | re.DOTALL)
            section_match = sec_regex.search(str(section_contents))
            if section_match:
                section_contents = section_match.group(1)
            page_template.section_contents = section_contents
                
            page_template.section_name = relative_path.capitalize()
            page_template.section_links = makeLinksHTML(mylinks)
            
            html_code = html_regex.search(str(page_template)).group(1)
        else:
            
            page_template.contents._content = contents
            html_code = html_regex.search(str(page_template)).group(1)
        
        if string.find(str(page_template), "#BeginTemplate") == -1:
            page_template = '<html><!-- #BeginTemplate "/Templates/section_template.dwt" -->' + html_code + '<!-- #EndTemplate --></html>'

        page_template = title_regex.sub(title, str(page_template))
        if relative_path != "":
            page_template = page_template.replace('href="images', 'href="../images')
            page_template = page_template.replace('src="images', 'src="../images')
            
        page_template = adjustDWTemplate(page_template)
            
        output = open(filename, "w")
        output.write(str(page_template))
        output.close()
    except:
        # just move onto the next file
        sys.stderr.write("WARNING: Unable to update %s" % (filename))
        import traceback
        sys.stderr.write(`traceback.print_exc()`)

print "Updating site..."
# Now, update all the pages in the site with this template
updateDir(root_dir)
